searchNodes=[{"doc":"A blazing fast compile-time optimized alternative to the Enum and Stream modules. Overview Iter allows you to effortlessly write highly-efficient pipelines to process enumerables, in a familiar and highly readable style. iex&gt; require Iter iex&gt; 1 .. 10 |&gt; Iter . map ( &amp; &amp;1 * * 2 ) |&gt; Iter . sum ( ) 385 Iter will merge both the map and sum steps and perform both in one single pass. Unlike the same pipeline written with Enum , it won't build any intermediate list, therefore saving memory and CPU cycles. You can think of Iter as compile-time streams, or as comprehensions on steroids. It should be highly efficient compared to the same pipeline written with Stream , since it does most of the work at compile time without any runtime overhead. And while it actually works very similarly to for/1 under the hood and basically emits the same code, it offers a much more flexible, composable and extensive API. The benchmarks folder illustrates how Iter compares to Enum or Stream through some examples. Because Iter is compile-time, these are macros and not functions. This has several implications: you have to require the module first before using it they won't appear in the stacktrace in case of errors (but Iter tries to make sure that stacktraces will point to the line of the step responsible) if you &quot;break&quot; the pipeline, Iter won't be able to optimize it as a single pass: it will suffer the same issue as Enum 1 .. 10 |&gt; Iter . map ( &amp; &amp;1 * * 2 ) |&gt; IO . inspect ( ) # &lt;= pipeline broken, creates an intermediate list |&gt; Iter . sum ( ) When there is no possibility of merging steps, Iter is simply delegating to Enum which is optimized plenty on individual steps. Consistency Iter is mostly consistent with the standard library, but it is prioritizing efficiency over absolute consistency with the Enum and Stream modules, which implies some slight differences. These differences are always documented in the concerned macro docs. Negative indexes Iter only supports positive indexes when inside a pipeline, so most of functions like at/1 , slice/1 or take/1 which would also accept negative indexes cannot be replaced in cases needing it. The reason is simple: working with negative indexes implies to materialize the whole list once. If you need it, you should replace the relevant step to use Enum , or maybe call Iter.reverse/1 before accessing it (see Collecting the pipeline section). API differences Iter should cover most of the Enum API, but: some operations are still missing some operations won't be added because cannot be implemented efficiently some extra functions are being provided: Iter.mean/1 , Iter.first/1 , Iter.last/1 (to compensate with the lack of negative indexes) and Iter.match/3 (pattern-match to filter and extract at once, like in comprehensions) Collecting the pipeline Some operations like Iter.to_list/1 , Iter.reverse/2 , Iter.reduce/3 , Iter.group_by/2 ... need to materialize an intermediate list or accumulator and will collect the pipeline. Operations that are collecting the pipeline are always mentioning it in their documentation. Here is a simple example: users |&gt; Iter . map ( &amp; fetch_user / 1 ) |&gt; Iter . reject ( &amp; is_nil / 1 ) |&gt; Iter . each ( &amp; process_user / 1 ) The pipeline above will start processing users as they are retrieved, in one single pass. But assuming we want to make sure to be able to first retrieving all users before starting the processing step, Iter.to_list/1 can be used to make the intent explicit: users |&gt; Iter . map ( &amp; fetch_user / 1 ) |&gt; Iter . reject ( &amp; is_nil / 1 ) |&gt; Iter . to_list ( ) # forcing the pipeline to collect |&gt; Iter . each ( &amp; process_user / 1 ) Forcing a pipeline to collect through Enum.to_list/1 (or Enum.reverse/1 , faster) can also be used to circumvent some of Iter 's limitations like the absence of negative indexes support: foo |&gt; Iter . map ( &amp; bar / 1 ) |&gt; Iter . to_list ( ) # forcing the pipeline to collect |&gt; Iter . take ( maybe_negative_index ) In the example above, Iter.take/2 is now the only step of its pipeline and can support negative indexing. The extra pass required is made explicit.","ref":"Iter.html","title":"Iter","type":"module"},{"doc":"Returns true if all elements in enumerable are truthy. Equivalent to Enum.all?/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . all? ( [ &quot;yes&quot; , true ] ) true iex&gt; Iter . all? ( [ false , true ] ) false iex&gt; Iter . all? ( [ ] ) true","ref":"Iter.html#all?/1","title":"Iter.all?/1","type":"macro"},{"doc":"Returns true if fun returns a truthy value for all elements in enumerable . Equivalent to Enum.all?/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Enum . all? ( [ 2 , 4 , 6 ] , fn x -&gt; rem ( x , 2 ) == 0 end ) true iex&gt; Enum . all? ( [ 2 , 3 , 4 ] , fn x -&gt; rem ( x , 2 ) == 0 end ) false iex&gt; Iter . all? ( [ ] , fn x -&gt; rem ( x , 2 ) == 0 end ) true","ref":"Iter.html#all?/2","title":"Iter.all?/2","type":"macro"},{"doc":"Returns true if at least one element in enumerable is truthy. Equivalent to Enum.any?/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . any? ( [ false , true ] ) true iex&gt; Iter . any? ( [ false , nil ] ) false iex&gt; Iter . any? ( [ ] ) false","ref":"Iter.html#any?/1","title":"Iter.any?/1","type":"macro"},{"doc":"Returns true if fun returns a truthy value for at least one element in enumerable . Equivalent to Enum.any?/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Enum . any? ( [ 2 , 4 , 6 ] , fn x -&gt; rem ( x , 2 ) == 1 end ) false iex&gt; Enum . any? ( [ 2 , 3 , 4 ] , fn x -&gt; rem ( x , 2 ) == 1 end ) true iex&gt; Iter . any? ( [ ] , fn x -&gt; rem ( x , 2 ) == 1 end ) false","ref":"Iter.html#any?/2","title":"Iter.any?/2","type":"macro"},{"doc":"Finds the element at the given index (zero-based). Equivalent to Enum.at/2 . Returns nil if not found. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Note: Negative indexes are NOT supported when used in a pipeline, since this would imply to materialize the whole list and therefore cannot be done lazily. If you need to use negative indexes, you can either use materialize the pipeline first using Iter.to_list/1 or use the equivalent Enum function. Read the Collecting the pipeline section for more information. Also, see Iter.last/1 if you need to access the last element. Examples iex&gt; Iter . at ( [ :foo , :bar , :baz ] , 2 ) :baz iex&gt; Iter . at ( [ :foo , :bar , :baz ] , 3 ) nil","ref":"Iter.html#at/2","title":"Iter.at/2","type":"macro"},{"doc":"Finds the element at the given index (zero-based). Equivalent to Enum.at/3 . Returns default if not found. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Note: Negative indexes are NOT supported when used in a pipeline, since this would imply to materialize the whole list and therefore cannot be done lazily. If you need to use negative indexes, you can either use materialize the pipeline first using Iter.to_list/1 or use the equivalent Enum function. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . at ( 1 .. 1000 , 5 , :none ) 6 iex&gt; Iter . at ( 1 .. 1000 , 1000 , :none ) :none","ref":"Iter.html#at/3","title":"Iter.at/3","type":"macro"},{"doc":"Given an enumerable of enumerables, concatenates the enumerables into a single one. Equivalent to Enum.concat/1 . Examples iex&gt; Iter . concat ( [ 1 .. 3 , 4 .. 6 ] ) [ 1 , 2 , 3 , 4 , 5 , 6 ]","ref":"Iter.html#concat/1","title":"Iter.concat/1","type":"macro"},{"doc":"Concatenates the enumerable on the right with the enumerable on the left . Equivalent to Enum.concat/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . concat ( 1 .. 3 , 4 .. 6 ) [ 1 , 2 , 3 , 4 , 5 , 6 ]","ref":"Iter.html#concat/2","title":"Iter.concat/2","type":"macro"},{"doc":"Returns the size of the enumerable. Equivalent to Enum.count/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . count ( [ 1 , 2 , 3 ] ) 3","ref":"Iter.html#count/1","title":"Iter.count/1","type":"macro"},{"doc":"Returns the count of elements in the enumerable for which fun returns a truthy value. Equivalent to Enum.count/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . count ( 1 .. 5 , fn x -&gt; rem ( x , 2 ) == 0 end ) 2","ref":"Iter.html#count/2","title":"Iter.count/2","type":"macro"},{"doc":"Enumerates the enumerable , removing successive duplicate elements. Equivalent to Enum.dedup/1 . Examples iex&gt; Iter . dedup ( [ 1 , 2 , 2 , 3 , 3 , 1 , 3 ] ) [ 1 , 2 , 3 , 1 , 3 ]","ref":"Iter.html#dedup/1","title":"Iter.dedup/1","type":"macro"},{"doc":"Enumerates the enumerable , removing successive elements for which fun return duplicate values. Equivalent to Enum.dedup_by/2 . Examples iex&gt; Iter . dedup_by ( [ { 1 , :a } , { 2 , :b } , { 2 , :c } , { 1 , :a } ] , fn { x , _ } -&gt; x end ) [ { 1 , :a } , { 2 , :b } , { 1 , :a } ]","ref":"Iter.html#dedup_by/2","title":"Iter.dedup_by/2","type":"macro"},{"doc":"Drops an amount of elements from the beginning of the enumerable . Equivalent to Enum.drop/2 . Note: Negative indexes are NOT supported when used in a pipeline, since this would imply to materialize the whole list and therefore cannot be done lazily. If you need to use negative indexes, you can either use materialize the pipeline first using Iter.to_list/1 or use the equivalent Enum function. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . drop ( 1 .. 10 , 5 ) [ 6 , 7 , 8 , 9 , 10 ]","ref":"Iter.html#drop/2","title":"Iter.drop/2","type":"macro"},{"doc":"Returns a list of every nth element in the enumerable dropped, starting with the first element. Equivalent to Enum.drop_every/2 . Examples iex&gt; Iter . drop_every ( 1 .. 10 , 3 ) [ 2 , 3 , 5 , 6 , 8 , 9 ]","ref":"Iter.html#drop_every/2","title":"Iter.drop_every/2","type":"macro"},{"doc":"Drops elements at the beginning of the enumerable , while fun returns a truthy value. Equivalent to Enum.drop_while/2 . Examples iex&gt; Iter . drop_while ( 1 .. 10 , &amp; &amp;1 &lt; 6 ) [ 6 , 7 , 8 , 9 , 10 ]","ref":"Iter.html#drop_while/2","title":"Iter.drop_while/2","type":"macro"},{"doc":"Invokes the given fun for each element in the enumerable . Equivalent to Enum.each/2 . Returns :ok . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; { :ok , pid } = Agent . start ( fn -&gt; [ ] end ) iex&gt; Iter . each ( 1 .. 5 , fn i -&gt; Agent . update ( pid , &amp; [ i | &amp;1 ] ) end ) :ok iex&gt; Agent . get ( pid , &amp; &amp;1 ) [ 5 , 4 , 3 , 2 , 1 ]","ref":"Iter.html#each/2","title":"Iter.each/2","type":"macro"},{"doc":"Returns true if enumerable is empty, otherwise false . Equivalent to Enum.empty?/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . empty? ( [ ] ) true iex&gt; Iter . empty? ( [ :foo ] ) false","ref":"Iter.html#empty?/1","title":"Iter.empty?/1","type":"macro"},{"doc":"Finds the element at the given index (zero-based). Equivalent to Enum.fetch/2 . Returns {:ok, element} if found, otherwise :error . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Note: Negative indexes are NOT supported when used in a pipeline, since this would imply to materialize the whole list and therefore cannot be done lazily. If you need to use negative indexes, you can either use materialize the pipeline first using Iter.to_list/1 or use the equivalent Enum function. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . fetch ( [ :foo , :bar , :baz ] , 2 ) { :ok , :baz } iex&gt; Iter . fetch ( [ :foo , :bar , :baz ] , 3 ) :error","ref":"Iter.html#fetch/2","title":"Iter.fetch/2","type":"macro"},{"doc":"Finds the element at the given index (zero-based). Equivalent to Enum.fetch!/2 . Raises OutOfBoundsError if the given index is outside the range of the enumerable . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Note: Negative indexes are NOT supported when used in a pipeline, since this would imply to materialize the whole list and therefore cannot be done lazily. If you need to use negative indexes, you can either use materialize the pipeline first using Iter.to_list/1 or use the equivalent Enum function. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . fetch! ( [ :foo , :bar , :baz ] , 2 ) :baz iex&gt; Iter . fetch! ( [ :foo , :bar , :baz ] , 3 ) ** (Enum.OutOfBoundsError) out of bounds error","ref":"Iter.html#fetch!/2","title":"Iter.fetch!/2","type":"macro"},{"doc":"Filters the enumerable , keeping only elements for which fun returns a truthy value. Equivalent to Enum.filter/2 . Examples iex&gt; Iter . filter ( 1 .. 4 , &amp; rem ( &amp;1 , 2 ) == 1 ) [ 1 , 3 ]","ref":"Iter.html#filter/2","title":"Iter.filter/2","type":"macro"},{"doc":"Returns the first element for which fun returns a truthy value. Equivalent to Enum.find/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . find ( [ 2 , 3 , 4 ] , fn x -&gt; rem ( x , 2 ) == 1 end ) 3 iex&gt; Iter . find ( [ 2 , 4 , 6 ] , fn x -&gt; rem ( x , 2 ) == 1 end ) nil","ref":"Iter.html#find/2","title":"Iter.find/2","type":"macro"},{"doc":"Returns the first element for which fun returns a truthy value, returns default if not found.any() Equivalent to Enum.find/3 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . find ( [ 2 , 3 , 4 ] , 0 , fn x -&gt; rem ( x , 2 ) == 1 end ) 3 iex&gt; Iter . find ( [ 2 , 4 , 6 ] , 0 , fn x -&gt; rem ( x , 2 ) == 1 end ) 0","ref":"Iter.html#find/3","title":"Iter.find/3","type":"macro"},{"doc":"Similar to find/2 , but returns the index (zero-based) of the element instead of the element itself. Equivalent to Enum.find_index/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . find_index ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , fn x -&gt; x =~ &quot;b&quot; end ) 1 iex&gt; Iter . find_index ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , fn x -&gt; x =~ &quot;z&quot; end ) nil","ref":"Iter.html#find_index/2","title":"Iter.find_index/2","type":"macro"},{"doc":"Similar to find/2 , but returns the value of the function invocation instead of the element itself. Equivalent to Enum.find_value/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . find_value ( [ %{ x : nil } , %{ x : 5 } , %{ } ] , fn map -&gt; map [ :x ] end ) 5 iex&gt; Iter . find_value ( [ %{ x : nil } , %{ } , %{ } ] , fn map -&gt; map [ :x ] end ) nil","ref":"Iter.html#find_value/2","title":"Iter.find_value/2","type":"macro"},{"doc":"Similar to find/3 , but returns the value of the function invocation instead of the element itself. Equivalent to Enum.find_value/3 . Returns default if not found. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . find_value ( [ %{ x : nil } , %{ x : 5 } , %{ } ] , 0 , fn map -&gt; map [ :x ] end ) 5 iex&gt; Iter . find_value ( [ %{ x : nil } , %{ } , %{ } ] , 0 , fn map -&gt; map [ :x ] end ) 0","ref":"Iter.html#find_value/3","title":"Iter.find_value/3","type":"macro"},{"doc":"Retrieves the first element of the enumerable , or nil if empty. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. There is no equivalent Enum function. Examples iex&gt; Iter . first ( 1 .. 1000 ) 1 iex&gt; Iter . first ( [ ] ) nil","ref":"Iter.html#first/1","title":"Iter.first/1","type":"macro"},{"doc":"Retrieves the first element of the enumerable , or default if empty. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. There is no equivalent Enum function. Examples iex&gt; Iter . first ( 1 .. 10 , :none ) 1 iex&gt; Iter . first ( [ ] , :none ) :none","ref":"Iter.html#first/2","title":"Iter.first/2","type":"macro"},{"doc":"Maps the given fun over enumerable and flattens the result. Equivalent to Enum.flat_map/2 . Examples iex&gt; Iter . flat_map ( 1 .. 3 , fn n -&gt; 1 .. n end ) [ 1 , 1 , 2 , 1 , 2 , 3 ]","ref":"Iter.html#flat_map/2","title":"Iter.flat_map/2","type":"macro"},{"doc":"Returns a map with keys as unique elements of enumerable and values as the count of every element. Equivalent to Enum.frequencies/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . frequencies ( [ 1 , 1 , 2 , 1 , 2 , 3 ] ) %{ 1 =&gt; 3 , 2 =&gt; 2 , 3 =&gt; 1 }","ref":"Iter.html#frequencies/1","title":"Iter.frequencies/1","type":"macro"},{"doc":"Returns a map with keys as unique elements given by key_fun and values as the count of every element. Equivalent to Enum.frequencies_by/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . frequencies_by ( ~w{aa aA bb cc} , &amp; String . downcase / 1 ) %{ &quot;aa&quot; =&gt; 2 , &quot;bb&quot; =&gt; 1 , &quot;cc&quot; =&gt; 1 }","ref":"Iter.html#frequencies_by/2","title":"Iter.frequencies_by/2","type":"macro"},{"doc":"Splits the enumerable into groups based on key_fun . Equivalent to Enum.group_by/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . group_by ( ~w{ant buffalo cat dingo} , &amp; String . length / 1 ) %{ 3 =&gt; [ &quot;ant&quot; , &quot;cat&quot; ] , 5 =&gt; [ &quot;dingo&quot; ] , 7 =&gt; [ &quot;buffalo&quot; ] }","ref":"Iter.html#group_by/2","title":"Iter.group_by/2","type":"macro"},{"doc":"Splits the enumerable into groups based on key_fun . Equivalent to Enum.group_by/3 . The result is a map where each key is given by key_fun and each value is a list of elements given by value_fun . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . group_by ( ~w{ant buffalo cat dingo} , &amp; String . length / 1 , &amp; String . first / 1 ) %{ 3 =&gt; [ &quot;a&quot; , &quot;c&quot; ] , 5 =&gt; [ &quot;d&quot; ] , 7 =&gt; [ &quot;b&quot; ] }","ref":"Iter.html#group_by/3","title":"Iter.group_by/3","type":"macro"},{"doc":"Intersperses separator between each element of the enumerable . Equivalent to Enum.intersperse/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . intersperse ( 1 .. 3 , :foo ) [ 1 , :foo , 2 , :foo , 3 ]","ref":"Iter.html#intersperse/2","title":"Iter.intersperse/2","type":"macro"},{"doc":"Inserts the given enumerable into a collectable . Equivalent to Enum.into/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . into ( [ a : 1 , b : 2 , a : 3 ] , %{ } ) %{ a : 3 , b : 2 } iex&gt; Iter . into ( [ 1 , 2 , 1 , 2.0 ] , MapSet . new ( ) ) MapSet . new ( [ 1 , 2 , 2.0 ] )","ref":"Iter.html#into/2","title":"Iter.into/2","type":"macro"},{"doc":"Inserts the given enumerable into a collectable and maps the fun function on each item. Equivalent to Enum.into/3 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . into ( [ a : 1 , b : 2 , a : 3 ] , %{ } , fn { k , v } -&gt; { k , v * 2 } end ) %{ a : 6 , b : 4 } iex&gt; Iter . into ( [ 1 , 2 , 1 , 2.0 ] , MapSet . new ( ) , &amp; &amp;1 * 2 ) MapSet . new ( [ 2 , 4 , 4.0 ] )","ref":"Iter.html#into/3","title":"Iter.into/3","type":"macro"},{"doc":"Joins the given enumerable into a string without any separator. Equivalent to Enum.join/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . join ( 1 .. 3 ) &quot;123&quot;","ref":"Iter.html#join/1","title":"Iter.join/1","type":"macro"},{"doc":"Joins the given enumerable into a string with joiner as a separator. Equivalent to Enum.join/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . join ( 1 .. 3 , &quot;-&quot; ) &quot;1-2-3&quot;","ref":"Iter.html#join/2","title":"Iter.join/2","type":"macro"},{"doc":"Retrieves the last element of the enumerable , or nil if empty. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. There is no equivalent Enum function, but it can compensate for the lack of negative index support in at/2 . Examples iex&gt; Iter . last ( 1 .. 10 ) 10 iex&gt; Iter . last ( [ ] ) nil","ref":"Iter.html#last/1","title":"Iter.last/1","type":"macro"},{"doc":"Retrieves the last element of the enumerable , or default if empty. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. There is no equivalent Enum function, but it can compensate for the lack of negative index support in at/3 . Examples iex&gt; Iter . last ( 1 .. 10 , :none ) 10 iex&gt; Iter . last ( [ ] , :none ) :none","ref":"Iter.html#last/2","title":"Iter.last/2","type":"macro"},{"doc":"Applies fun on each element of enumerable . Equivalent to Enum.map/2 . Examples iex&gt; Iter . map ( 1 .. 3 , &amp; &amp;1 * * 2 ) [ 1 , 4 , 9 ]","ref":"Iter.html#map/2","title":"Iter.map/2","type":"macro"},{"doc":"Maps and intersperses the given enumerable with separator . Equivalent to Enum.map_intersperse/3 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . map_intersperse ( 1 .. 3 , :foo , &amp; &amp;1 * * 2 ) [ 1 , :foo , 4 , :foo , 9 ]","ref":"Iter.html#map_intersperse/3","title":"Iter.map_intersperse/3","type":"macro"},{"doc":"Applies mapper and joins the given enumerable into a string without any separator. Equivalent to Enum.map_join/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . map_join ( 1 .. 3 , &amp; &amp;1 * * 2 ) &quot;149&quot;","ref":"Iter.html#map_join/2","title":"Iter.map_join/2","type":"macro"},{"doc":"Applies mapper and joins the given enumerable into a string with joiner as a separator. Equivalent to Enum.map_join/3 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . map_join ( 1 .. 3 , &quot;-&quot; , &amp; &amp;1 * * 2 ) &quot;1-4-9&quot;","ref":"Iter.html#map_join/3","title":"Iter.map_join/3","type":"macro"},{"doc":"Invokes the given function to each element in the enumerable to reduce it to a single element, while keeping an accumulator. Equivalent to Enum.map_reduce/3 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . map_reduce ( [ 1 , 2 , 3 ] , 0 , fn x , acc -&gt; { x * 2 , x + acc } end ) { [ 2 , 4 , 6 ] , 6 }","ref":"Iter.html#map_reduce/3","title":"Iter.map_reduce/3","type":"macro"},{"doc":"Pattern-matches on each element of the enumerable , filters out elements that do not match the pattern , and returns the expr . This works a bit like a combination of map/2 and filter/2 and works very similarly as for/1 comprehensions. There is no equivalent Enum function. Examples iex&gt; Iter . match ( [ { :ok , 1 } , :error , { :ok , 3 } ] , { :ok , x } , x + 1 ) [ 2 , 4 ] The pattern also supports guards: iex&gt; Iter . match ( [ 1 , nil , 3 ] , x when is_integer ( x ) , x * 2 ) [ 2 , 6 ]","ref":"Iter.html#match/3","title":"Iter.match/3","type":"macro"},{"doc":"Returns the maximal element in the enumerable according to Erlang's term ordering. Equivalent to Enum.max/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . max ( [ 2 , 4 , 1 , 3 ] ) 4 iex&gt; Iter . max ( [ ] ) ** (Enum.EmptyError) empty error","ref":"Iter.html#max/1","title":"Iter.max/1","type":"macro"},{"doc":"Returns the mean value of all elements in enumerable . Raises Enum.EmptyError if enumerable is empty. There is no equivalent Enum function. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . mean ( 1 .. 10 ) 5.5 iex&gt; Iter . mean ( [ ] ) ** (Enum.EmptyError) empty error","ref":"Iter.html#mean/1","title":"Iter.mean/1","type":"macro"},{"doc":"Checks if element exists within the enumerable . Equivalent to Enum.member?/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Enum . member? ( [ :ant , :bat , :cat ] , :bat ) true iex&gt; Enum . member? ( [ :ant , :bat , :cat ] , :dog ) false iex&gt; Iter . member? ( [ 1 , 2 , 3 ] , 2.0 ) false","ref":"Iter.html#member?/2","title":"Iter.member?/2","type":"macro"},{"doc":"Returns the minimal element in the enumerable according to Erlang's term ordering. Equivalent to Enum.min/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . min ( [ 2 , 4 , 1 , 3 ] ) 1 iex&gt; Iter . min ( [ ] ) ** (Enum.EmptyError) empty error","ref":"Iter.html#min/1","title":"Iter.min/1","type":"macro"},{"doc":"Returns the product of all elements in enumerable . Equivalent to Enum.product/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . product ( 1 .. 3 ) 6","ref":"Iter.html#product/1","title":"Iter.product/1","type":"macro"},{"doc":"Returns a random element of the enumerable . Equivalent to Enum.random/1 . Raises Enum.EmptyError if enumerable is empty. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples # Although not necessary, let&#39;s seed the random algorithm iex&gt; :rand . seed ( :exsss , { 1 , 2 , 3 } ) iex&gt; Iter . random ( 1 .. 100 ) 27","ref":"Iter.html#random/1","title":"Iter.random/1","type":"macro"},{"doc":"Invokes fun for each element in the enumerable with the accumulator. Equivalent to Enum.reduce/2 . Raises Enum.EmptyError if enumerable is empty. The first element of the enumerable is used as the initial value of the accumulator. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . reduce ( 1 .. 5 , &amp; * / 2 ) 120 iex&gt; Iter . reduce ( [ ] , &amp; * / 2 ) ** (Enum.EmptyError) empty error","ref":"Iter.html#reduce/2","title":"Iter.reduce/2","type":"macro"},{"doc":"Invokes fun for each element in the enumerable with the accumulator. Equivalent to Enum.reduce/3 . The value of acc is used as the initial value of the accumulator. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . reduce ( 1 .. 5 , 1 , &amp; * / 2 ) 120","ref":"Iter.html#reduce/3","title":"Iter.reduce/3","type":"macro"},{"doc":"Filters the enumerable , rejecting elements for which fun returns a truthy value. Equivalent to Enum.reject/2 . Examples iex&gt; Iter . reject ( 1 .. 4 , &amp; rem ( &amp;1 , 2 ) == 1 ) [ 2 , 4 ]","ref":"Iter.html#reject/2","title":"Iter.reject/2","type":"macro"},{"doc":"Returns a list of elements in enumerable in reverse order. Equivalent to Enum.reverse/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . reverse ( 1 .. 3 ) [ 3 , 2 , 1 ]","ref":"Iter.html#reverse/1","title":"Iter.reverse/1","type":"macro"},{"doc":"Reverses the elements in enumerable , appends the tail , and returns the result as a list. Equivalent to Enum.reverse/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . reverse ( 1 .. 3 , 4 .. 6 ) [ 3 , 2 , 1 , 4 , 5 , 6 ]","ref":"Iter.html#reverse/2","title":"Iter.reverse/2","type":"macro"},{"doc":"Applies the given function to each element in the enumerable , storing the result in a list and passing it as the accumulator for the next computation. Equivalent to Enum.scan/3 . Examples iex&gt; Iter . scan ( 1 .. 5 , 1 , &amp; * / 2 ) [ 1 , 2 , 6 , 24 , 120 ]","ref":"Iter.html#scan/3","title":"Iter.scan/3","type":"macro"},{"doc":"Returns a list with enumerable elements in a random order. Equivalent to Enum.shuffle/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples # Although not necessary, let&#39;s seed the random algorithm iex&gt; :rand . seed ( :exsss , { 1 , 2 , 3 } ) iex&gt; Iter . shuffle ( 1 .. 6 ) [ 3 , 2 , 5 , 1 , 4 , 6 ]","ref":"Iter.html#shuffle/1","title":"Iter.shuffle/1","type":"macro"},{"doc":"Returns a subset list of the given enumerable by index_range . Equivalent to Enum.slice/2 . Note: Negative indexes are NOT supported when used in a pipeline, since this would imply to materialize the whole list and therefore cannot be done lazily. If you need to use negative indexes, you can either use materialize the pipeline first using Iter.to_list/1 or use the equivalent Enum function. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . slice ( 1 .. 100 , 5 .. 15 ) [ 6 , 7 , 8 , 9 , 10 , 11 , 12 , 13 , 14 , 15 , 16 ] iex&gt; Iter . slice ( 1 .. 100 , 5 .. 15 // 5 ) [ 6 , 11 , 16 ]","ref":"Iter.html#slice/2","title":"Iter.slice/2","type":"macro"},{"doc":"Returns a subset list of the given enumerable, from start_index with amount number of elements if available. Equivalent to Enum.slice/3 . Note: Negative indexes are NOT supported when used in a pipeline, since this would imply to materialize the whole list and therefore cannot be done lazily. If you need to use negative indexes, you can either use materialize the pipeline first using Iter.to_list/1 or use the equivalent Enum function. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . slice ( 1 .. 100 , 5 , 10 ) [ 6 , 7 , 8 , 9 , 10 , 11 , 12 , 13 , 14 , 15 ]","ref":"Iter.html#slice/3","title":"Iter.slice/3","type":"macro"},{"doc":"Sorts the enumerable according to Erlang's term ordering. Equivalent to Enum.sort/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . sort ( [ 4 , 1 , 5 , 2 , 3 ] ) [ 1 , 2 , 3 , 4 , 5 ]","ref":"Iter.html#sort/1","title":"Iter.sort/1","type":"macro"},{"doc":"Sorts the enumerable by the given sorter function or module. Equivalent to Enum.sort/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . sort ( [ 4 , 1 , 5 , 2 , 3 ] , :desc ) [ 5 , 4 , 3 , 2 , 1 ]","ref":"Iter.html#sort/2","title":"Iter.sort/2","type":"macro"},{"doc":"Sorts the mapped results of the enumerable according to Erlang's term ordering. Equivalent to Enum.sort_by/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. This is actually just an alias for Enum.sort_by/2 , Iter isn't able to optimize it. Examples iex&gt; Iter . sort_by ( [ &quot;some&quot; , &quot;kind&quot; , &quot;of&quot; , &quot;monster&quot; ] , &amp; byte_size / 1 ) [ &quot;of&quot; , &quot;some&quot; , &quot;kind&quot; , &quot;monster&quot; ]","ref":"Iter.html#sort_by/3","title":"Iter.sort_by/3","type":"function"},{"doc":"Splits the enumerable into two lists, leaving amount elements in the first one. Equivalent to Enum.split/2 . Note: Negative indexes are NOT supported when used in a pipeline, since this would imply to materialize the whole list and therefore cannot be done lazily. If you need to use negative indexes, you can either use materialize the pipeline first using Iter.to_list/1 or use the equivalent Enum function. Read the Collecting the pipeline section for more information. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . split ( 1 .. 10 , 5 ) { [ 1 , 2 , 3 , 4 , 5 ] , [ 6 , 7 , 8 , 9 , 10 ] }","ref":"Iter.html#split/2","title":"Iter.split/2","type":"macro"},{"doc":"Splits the enumerable in two at the position of the element for which fun returns a falsy value for the first time. Equivalent to Enum.split_while/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . split_while ( 1 .. 10 , &amp; &amp;1 &lt; 6 ) { [ 1 , 2 , 3 , 4 , 5 ] , [ 6 , 7 , 8 , 9 , 10 ] }","ref":"Iter.html#split_while/2","title":"Iter.split_while/2","type":"macro"},{"doc":"Splits the enumerable in two lists based on the truthiness of applying fun on each element. Equivalent to Enum.split_with/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . split_with ( 1 .. 4 , &amp; rem ( &amp;1 , 2 ) == 1 ) { [ 1 , 3 ] , [ 2 , 4 ] }","ref":"Iter.html#split_with/2","title":"Iter.split_with/2","type":"macro"},{"doc":"Returns the sum of all elements in enumerable . Equivalent to Enum.sum/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . sum ( 1 .. 3 ) 6","ref":"Iter.html#sum/1","title":"Iter.sum/1","type":"macro"},{"doc":"Takes an amount of elements from the beginning of the enumerable . Equivalent to Enum.take/2 . Note: Negative indexes are NOT supported when used in a pipeline, since this would imply to materialize the whole list and therefore cannot be done lazily. If you need to use negative indexes, you can either use materialize the pipeline first using Iter.to_list/1 or use the equivalent Enum function. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . take ( 1 .. 1000 , 5 ) [ 1 , 2 , 3 , 4 , 5 ]","ref":"Iter.html#take/2","title":"Iter.take/2","type":"macro"},{"doc":"Returns a list of every nth element in the enumerable , starting with the first element. Equivalent to Enum.take_every/2 . Examples iex&gt; Iter . take_every ( 1 .. 10 , 3 ) [ 1 , 4 , 7 , 10 ]","ref":"Iter.html#take_every/2","title":"Iter.take_every/2","type":"macro"},{"doc":"Takes an amount of random elements from the enumerable . Equivalent to Enum.take_random/2 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples # Although not necessary, let&#39;s seed the random algorithm iex&gt; :rand . seed ( :exsss , { 1 , 2 , 3 } ) iex&gt; Iter . take_random ( 1 .. 100 , 3 ) [ 74 , 28 , 55 ]","ref":"Iter.html#take_random/2","title":"Iter.take_random/2","type":"macro"},{"doc":"Takes the elements from the beginning of the enumerable , while fun returns a truthy value. Equivalent to Enum.take_while/2 . Examples iex&gt; Iter . take_while ( 1 .. 1000 , &amp; &amp;1 &lt; 6 ) [ 1 , 2 , 3 , 4 , 5 ]","ref":"Iter.html#take_while/2","title":"Iter.take_while/2","type":"macro"},{"doc":"Converts enumerable to a list. Equivalent to Enum.to_list/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . to_list ( 1 .. 3 ) [ 1 , 2 , 3 ]","ref":"Iter.html#to_list/1","title":"Iter.to_list/1","type":"macro"},{"doc":"Enumerates the enumerable , removing the duplicate elements. Equivalent to Enum.uniq/1 . Examples iex&gt; Iter . uniq ( [ 1 , 2 , 1 , 3 , 2 , 4 ] ) [ 1 , 2 , 3 , 4 ]","ref":"Iter.html#uniq/1","title":"Iter.uniq/1","type":"macro"},{"doc":"Enumerates the enumerable , removing elements for which fun return duplicate values. Equivalent to Enum.uniq_by/2 . Examples iex&gt; Iter . uniq_by ( [ { 1 , :x } , { 2 , :y } , { 1 , :z } ] , fn { x , _ } -&gt; x end ) [ { 1 , :x } , { 2 , :y } ]","ref":"Iter.html#uniq_by/2","title":"Iter.uniq_by/2","type":"macro"},{"doc":"Extracts two-element tuples from the given enumerable and returns them as two separate lists. Equivalent to Enum.unzip/1 . Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . unzip ( [ { :a , 1 } , { :b , 2 } , { :c , 3 } ] ) { [ :a , :b , :c ] , [ 1 , 2 , 3 ] }","ref":"Iter.html#unzip/1","title":"Iter.unzip/1","type":"macro"},{"doc":"Returns the enumerable with each element wrapped in a tuple alongside its index. Equivalent to Enum.with_index/1 . Examples iex&gt; Iter . with_index ( [ &quot;a&quot; , &quot;b&quot; , &quot;c&quot; ] ) [ { &quot;a&quot; , 0 } , { &quot;b&quot; , 1 } , { &quot;c&quot; , 2 } ]","ref":"Iter.html#with_index/1","title":"Iter.with_index/1","type":"macro"},{"doc":"Returns the enumerable with each element wrapped in a tuple alongside its index. Equivalent to Enum.with_index/2 . Like Enum.with_index/2 , accepts either an anonymous function or an integer offset, but it has to infer the type at compile time. If the expression can't be inferred to be either an fn or a capture, it will assume it is an integer (example: Iter.with_index(list, var) will only work if var is an integer). If an offset is given, it will index from the given offset instead of from zero. If a function is given, it will index by invoking the function for each element and index (zero-based) of the enumerable . Examples iex&gt; Iter . with_index ( [ &quot;a&quot; , &quot;b&quot; , &quot;c&quot; ] , 100 ) [ { &quot;a&quot; , 100 } , { &quot;b&quot; , 101 } , { &quot;c&quot; , 102 } ] iex&gt; Iter . with_index ( [ &quot;a&quot; , &quot;b&quot; , &quot;c&quot; ] , fn elem , index -&gt; String . duplicate ( elem , index ) end ) [ &quot;&quot; , &quot;b&quot; , &quot;cc&quot; ] iex&gt; Iter . with_index ( [ &quot;a&quot; , &quot;b&quot; , &quot;c&quot; ] , &amp; String . duplicate ( &amp;1 , &amp;2 ) ) [ &quot;&quot; , &quot;b&quot; , &quot;cc&quot; ]","ref":"Iter.html#with_index/2","title":"Iter.with_index/2","type":"macro"},{"doc":"Zips corresponding elements from a finite collection of enumerables into a list of tuples. This is actually just an alias for Enum.zip/1 , Iter isn't able to optimize it. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . zip ( [ [ 1 , 2 , 3 ] , [ :a , :b , :c ] , [ &quot;foo&quot; , &quot;bar&quot; , &quot;baz&quot; ] ] ) [ { 1 , :a , &quot;foo&quot; } , { 2 , :b , &quot;bar&quot; } , { 3 , :c , &quot;baz&quot; } ]","ref":"Iter.html#zip/1","title":"Iter.zip/1","type":"function"},{"doc":"Zips corresponding elements from two enumerables into a list of tuples. This is actually just an alias for Enum.zip/2 , Iter isn't able to optimize it. Note: This step collects the pipeline and cannot be merged with following steps. Read the Collecting the pipeline section for more information. Examples iex&gt; Iter . zip ( [ 1 , 2 , 3 , 4 , 5 ] , [ :a , :b , :c ] ) [ { 1 , :a } , { 2 , :b } , { 3 , :c } ]","ref":"Iter.html#zip/2","title":"Iter.zip/2","type":"function"},{"doc":"A blazing fast compile-time optimized alternative to the Enum and Stream modules.","ref":"readme.html","title":"Iter","type":"extras"},{"doc":"Iter allows you to effortlessly write highly-efficient pipelines to process enumerables, in a familiar and highly readable style. iex&gt; require Iter iex&gt; 1 .. 10 |&gt; Iter . map ( &amp; &amp;1 * * 2 ) |&gt; Iter . sum ( ) 385 Iter will merge both the map and sum steps and perform both in one single pass. Unlike the same pipeline written with Enum , it won't build any intermediate list, therefore saving memory and CPU cycles. You can think of Iter as compile-time streams, or as comprehensions on steroids. It should be highly efficient compared to the same pipeline written with Stream , since it does most of the work at compile time without any runtime overhead. And while it actually works very similarly to for/1 under the hood and basically emits the same code, it offers a much more flexible, composable and extensive API. The benchmarks folder illustrates how Iter compares to Enum or Stream through some examples. Because Iter is compile-time, these are macros and not functions. This has several implications: you have to require the module first before using it they won't appear in the stacktrace in case of errors (but Iter tries to make sure that stacktraces will point to the line of the step responsible) if you &quot;break&quot; the pipeline, Iter won't be able to optimize it as a single pass: it will suffer the same issue as Enum 1 .. 10 |&gt; Iter . map ( &amp; &amp;1 * * 2 ) |&gt; IO . inspect ( ) # &lt;= pipeline broken, creates an intermediate list |&gt; Iter . sum ( ) When there is no possibility of merging steps, Iter is simply delegating to Enum which is optimized plenty on individual steps.","ref":"readme.html#overview","title":"Iter - Overview","type":"extras"},{"doc":"Iter can be installed by adding iter to your list of dependencies in mix.exs : def deps do [ { :iter , &quot;~&gt; 0.1.2&quot; } ] end The documentation can be found at https://hexdocs.pm/iter .","ref":"readme.html#installation","title":"Iter - Installation","type":"extras"},{"doc":"Iter aims to provide a production-ready alternative to the Enum and Stream modules, that allows to write enumerable pipelines in a familiar way without a need to be concerned about efficiency. Premature optimization is the root of all evil. But by consistently providing better performance out of the box, Iter aims to help focusing more on writing readable code, and to remove the tradeoff between readability and performance. While Iter is close to offer a drop-in replacement for Enum / Stream , it doesn't aim to be an exact one. The Consistency section below covers the differences with the standard library.","ref":"readme.html#motivation","title":"Iter - Motivation","type":"extras"},{"doc":"Iter is mostly consistent with the standard library, but it is prioritizing efficiency over absolute consistency with the Enum and Stream modules, which implies some slight differences. These differences are always documented in the concerned macro docs. Negative indexes Iter only supports positive indexes when inside a pipeline, so most of functions like at/1 , slice/1 or take/1 which would also accept negative indexes cannot be replaced in cases needing it. The reason is simple: working with negative indexes implies to materialize the whole list once. If you need it, you should replace the relevant step to use Enum , or maybe call Iter.reverse/1 before accessing it (see Collecting the pipeline section). API differences Iter should cover most of the Enum API, but: some operations are still missing some operations won't be added because cannot be implemented efficiently some extra functions are being provided: Iter.mean/1 , Iter.first/1 , Iter.last/1 (to compensate with the lack of negative indexes) and Iter.match/3 (pattern-match to filter and extract at once, like in comprehensions)","ref":"readme.html#consistency","title":"Iter - Consistency","type":"extras"},{"doc":"Some operations like Iter.to_list/1 , Iter.reverse/2 , Iter.reduce/3 , Iter.group_by/2 ... need to materialize an intermediate list or accumulator and will collect the pipeline. Operations that are collecting the pipeline are always mentioning it in their documentation. Here is a simple example: users |&gt; Iter . map ( &amp; fetch_user / 1 ) |&gt; Iter . reject ( &amp; is_nil / 1 ) |&gt; Iter . each ( &amp; process_user / 1 ) The pipeline above will start processing users as they are retrieved, in one single pass. But assuming we want to make sure to be able to first retrieving all users before starting the processing step, Iter.to_list/1 can be used to make the intent explicit: users |&gt; Iter . map ( &amp; fetch_user / 1 ) |&gt; Iter . reject ( &amp; is_nil / 1 ) |&gt; Iter . to_list ( ) # forcing the pipeline to collect |&gt; Iter . each ( &amp; process_user / 1 ) Forcing a pipeline to collect through Enum.to_list/1 (or Enum.reverse/1 , faster) can also be used to circumvent some of Iter 's limitations like the absence of negative indexes support: foo |&gt; Iter . map ( &amp; bar / 1 ) |&gt; Iter . to_list ( ) # forcing the pipeline to collect |&gt; Iter . take ( maybe_negative_index ) In the example above, Iter.take/2 is now the only step of its pipeline and can support negative indexing. The extra pass required is made explicit.","ref":"readme.html#collecting-the-pipeline","title":"Iter - Collecting the pipeline","type":"extras"},{"doc":"Still learning Enum and struggling to find the right function? Make sure to check our Ultimate Enum cheatsheet !","ref":"readme.html#resources","title":"Iter - Resources","type":"extras"},{"doc":"Iter is licensed under the MIT License .","ref":"readme.html#copyright-and-license","title":"Iter - Copyright and License","type":"extras"},{"doc":"The Enum module provides functions that are incredibly useful, flexible and expressive. It is fair to claim that it is more than making up for the lack of loops in Elixir. However, the number of functions might feel overwhelming at first. This cheatsheet is meant as an attempt to categorize functions in order to learn or find them more easily. It doesn't mean to be exhaustive, and it also contains a few non- Enum yet useful recipes for working with enumerables. Disclaimer: this cheatsheet is not directly related to Iter , but since Iter API is based on Enum it can also help its usage .","ref":"enum.html","title":"The Ultimate Enum Cheatsheet","type":"extras"},{"doc":"Transform each element Enum.map/2 iex&gt; Enum . map ( 1 .. 3 , &amp; &amp;1 * * 2 ) [ 1 , 4 , 9 ] Using the index: Enum.with_index/2 iex&gt; Enum . with_index ( [ &quot;a&quot; , &quot;b&quot; , &quot;c&quot; ] ) [ { &quot;a&quot; , 0 } , { &quot;b&quot; , 1 } , { &quot;c&quot; , 2 } ] iex&gt; Enum . with_index ( [ &quot;a&quot; , &quot;b&quot; , &quot;c&quot; ] , &amp; String . duplicate ( &amp;1 , &amp;2 ) ) [ &quot;&quot; , &quot;b&quot; , &quot;cc&quot; ] Using another list/enumerable: Enum.zip/2 or Enum.zip_with/3 iex&gt; Enum . zip ( [ &quot;i&quot; , &quot;x&quot; , &quot;c&quot; ] , [ 1 , 10 , 100 ] ) [ { &quot;i&quot; , 1 } , { &quot;x&quot; , 10 } , { &quot;c&quot; , 100 } ] iex&gt; Enum . zip_with ( [ &quot;i&quot; , &quot;x&quot; , &quot;c&quot; ] , [ 1 , 10 , 100 ] , &amp; &quot; \#{ &amp;1 } : \#{ &amp;2 } &quot; ) [ &quot;i: 1&quot; , &quot;x: 10&quot; , &quot;c: 100&quot; ] See also: Enum.zip/1 , Enum.zip_with/2 , Enum.unzip/1 Using an accumulator: Enum.map_reduce/3 # 1/1, 2/3, 3/6, 4/10 iex&gt; Enum . map_reduce ( 1 .. 4 , 0 , fn x , acc -&gt; ...&gt; new_acc = x + acc ...&gt; { x / new_acc , new_acc } ...&gt; end ) { [ 1.0 , 0.6666666666666666 , 0.5 , 0.4 ] , 10 } Just cast as a list Enum.to_list/1 iex&gt; Enum . to_list ( 1 .. 3 ) [ 1 , 2 , 3 ] iex&gt; Enum . to_list ( %{ &quot;foo&quot; =&gt; 1 , &quot;bar&quot; =&gt; 2 } ) [ { &quot;bar&quot; , 2 } , { &quot;foo&quot; , 1 } ] Reordering Reverse order: Enum.reverse/2 iex&gt; Enum . reverse ( [ :a , :b , :c ] ) [ :c , :b , :a ] Enum.sort/1 / Enum.sort/2 iex&gt; Enum . sort ( [ :b , :d , :a , :c ] ) [ :a , :b , :c , :d ] iex&gt; Enum . sort ( [ :b , :d , :a , :c ] , :desc ) [ :d , :c , :b , :a ] Enum.sort_by/2 / Enum.sort_by/3 iex&gt; Enum . sort_by ( [ &quot;abc&quot; , &quot;d&quot; , &quot;ef&quot; ] , &amp; String . length / 1 ) [ &quot;d&quot; , &quot;ef&quot; , &quot;abc&quot; ] iex&gt; Enum . sort_by ( [ &quot;abc&quot; , &quot;d&quot; , &quot;ef&quot; ] , &amp; String . length / 1 , :desc ) [ &quot;abc&quot; , &quot;ef&quot; , &quot;d&quot; ] ⚠️ WARNING - Pitfall when comparing structs, see doc iex&gt; dates = [ ~D[2019-01-01] , ~D[2020-03-02] , ~D[2019-06-06] ] iex&gt; Enum . sort ( dates ) [ ~D[2019-01-01] , ~D[2020-03-02] , ~D[2019-06-06] ] iex&gt; Enum . sort ( dates , Date ) [ ~D[2019-01-01] , ~D[2019-06-06] , ~D[2020-03-02] ] Random order: Enum.shuffle/1 iex&gt; Enum . shuffle ( [ :a , :b , :c , :d ] ) [ :c , :a , :d , :b ]","ref":"enum.html#return-a-list-of-the-same-size","title":"The Ultimate Enum Cheatsheet - Return a list of the same size","type":"extras"},{"doc":"Filtering on a condition Enum.filter/2 / Enum.reject/2 iex&gt; Enum . filter ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &amp; &amp;1 =~ &quot;at&quot; ) [ &quot;bat&quot; , &quot;cat&quot; ] iex&gt; Enum . reject ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &amp; &amp;1 =~ &quot;at&quot; ) [ &quot;ant&quot; ] Both at once: Enum.split_with/2 : iex&gt; Enum . split_with ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &amp; &amp;1 =~ &quot;at&quot; ) { [ &quot;bat&quot; , &quot;cat&quot; ] , [ &quot;ant&quot; ] } Filtering + transforming in one pass: for/1 comprehension iex&gt; for s &lt;- [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , s =~ &quot;at&quot; do ...&gt; String . capitalize ( s ) ...&gt; end [ &quot;Bat&quot; , &quot;Cat&quot; ] Slicing From a range: Enum.slice/2 iex&gt; Enum . slice ( [ 1 , 2 , 3 , 4 , 5 ] , 1 .. 3 ) [ 2 , 3 , 4 ] iex&gt; Enum . slice ( 1 .. 100 , 5 .. 25 // 5 ) [ 6 , 11 , 16 , 21 , 26 ] From an index and amount: Enum.slice/3 iex&gt; Enum . slice ( 1 .. 100 , 5 , 10 ) [ 6 , 7 , 8 , 9 , 10 , 11 , 12 , 13 , 14 , 15 ] Amount-based Keep an amount: Enum.take/2 iex&gt; Enum . take ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; , &quot;dog&quot; ] , 2 ) [ &quot;ant&quot; , &quot;bat&quot; ] # negative index: take from the end iex&gt; Enum . take ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; , &quot;dog&quot; ] , - 2 ) [ &quot;cat&quot; , &quot;dog&quot; ] Remove an amount: Enum.drop/2 iex&gt; Enum . drop ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; , &quot;dog&quot; ] , 2 ) [ &quot;cat&quot; , &quot;dog&quot; ] # negative index: drop from the end iex&gt; Enum . drop ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; , &quot;dog&quot; ] , - 2 ) [ &quot;ant&quot; , &quot;bat&quot; ] Both at once: Enum.split/2 iex&gt; Enum . split ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; , &quot;dog&quot; ] , 2 ) { [ &quot;ant&quot; , &quot;bat&quot; ] , [ &quot;cat&quot; , &quot;dog&quot; ] } Keep/drop every nth element Enum.take_every/2 iex&gt; Enum . take_every ( 1 .. 10 , 2 ) [ 1 , 3 , 5 , 7 , 9 ] Enum.drop_every/2 iex&gt; Enum . drop_every ( 1 .. 10 , 2 ) [ 2 , 4 , 6 , 8 , 10 ] Until a condition is met Enum.take_while/2 / Enum.drop_while/2 iex&gt; Enum . take_while ( [ &quot;bat&quot; , &quot;cat&quot; , &quot;dog&quot; ] , &amp; &amp;1 =~ &quot;at&quot; ) [ &quot;bat&quot; , &quot;cat&quot; ] iex&gt; Enum . drop_while ( [ &quot;bat&quot; , &quot;cat&quot; , &quot;dog&quot; ] , &amp; &amp;1 =~ &quot;at&quot; ) [ &quot;dog&quot; ] Both at once: Enum.split_while/2 : iex&gt; Enum . split_while ( [ &quot;bat&quot; , &quot;cat&quot; , &quot;dog&quot; ] , &amp; &amp;1 =~ &quot;at&quot; ) { [ &quot;bat&quot; , &quot;cat&quot; ] , [ &quot;dog&quot; ] } Removing duplicates All duplicates: Enum.uniq/1 / Enum.uniq_by/2 iex&gt; Enum . uniq ( [ 3 , 2 , 1 , 1 , 2 , 3 ] ) [ 3 , 2 , 1 ] iex&gt; Enum . uniq_by ( [ &quot;foo&quot; , &quot;bar&quot; , &quot;baz&quot; , &quot;foo&quot; ] , &amp; String . first / 1 ) [ &quot;foo&quot; , &quot;bar&quot; ] Successive duplicates only: Enum.dedup/1 / Enum.dedup_by/2 iex&gt; Enum . dedup ( [ 3 , 2 , 1 , 1 , 2 , 3 ] ) [ 3 , 2 , 1 , 2 , 3 ] iex&gt; Enum . dedup_by ( [ &quot;foo&quot; , &quot;bar&quot; , &quot;baz&quot; , &quot;foo&quot; ] , &amp; String . first / 1 ) [ &quot;foo&quot; , &quot;bar&quot; , &quot;foo&quot; ] Side note - Use a set: MapSet.new/1 / MapSet.new/2 When working with unique elements, maybe a list isn't the best data structure. If you don't care about the ordering but want to efficiently test for membership, you most likely want a MapSet . iex&gt; MapSet . new ( [ 3 , 2 , 1 , 1 , 2 , 3 ] ) MapSet . new ( [ 1 , 2 , 3 ] ) iex&gt; MapSet . new ( [ &quot;foo&quot; , &quot;bar&quot; , &quot;baz&quot; , &quot;foo&quot; ] , &amp; String . first / 1 ) MapSet . new ( [ &quot;b&quot; , &quot;f&quot; ] ) Random sample Enum.take_random/2 iex&gt; Enum . take_random ( [ :a , :b , :c , :d ] , 2 ) [ :c , :a ]","ref":"enum.html#return-a-shorter-list","title":"The Ultimate Enum Cheatsheet - Return a shorter list","type":"extras"},{"doc":"Element matching a condition These will return early at the first match. Enum.find/2 iex&gt; Enum . find ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &amp; &amp;1 =~ &quot;at&quot; ) &quot;bat&quot; With transformation: Enum.find_value/2 iex&gt; Enum . find_value ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &amp; &amp;1 =~ &quot;at&quot; &amp;&amp; String . upcase ( &amp;1 ) ) &quot;BAT&quot; Its index: Enum.find_index/2 iex&gt; Enum . find_index ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &amp; &amp;1 =~ &quot;at&quot; ) 1 An extreme value Enum.max/1 / Enum.max_by/2 iex&gt; Enum . max ( [ 2 , 5 , 3 , 1 , 4 ] ) 5 iex&gt; Enum . max_by ( [ &quot;a&quot; , &quot;bcd&quot; , &quot;ef&quot; ] , &amp; String . length / 1 ) &quot;bcd&quot; Enum.min/1 / Enum.min_by/2 iex&gt; Enum . min ( [ 2 , 5 , 3 , 1 , 4 ] ) 1 iex&gt; Enum . min_by ( [ &quot;a&quot; , &quot;bcd&quot; , &quot;ef&quot; ] , &amp; String . length / 1 ) &quot;a&quot; ⚠️ WARNING - Pitfall when comparing structs, see doc iex&gt; Enum . max ( [ ~D[2017-03-31] , ~D[2017-04-01] ] ) ~D[2017-03-31] iex&gt; Enum . max ( [ ~D[2017-03-31] , ~D[2017-04-01] ] , Date ) ~D[2017-04-01] Both at once: Enum.min_max/1 / Enum.min_max_by/2 iex&gt; Enum . min_max ( [ 2 , 5 , 3 , 1 , 4 ] ) { 1 , 5 } iex&gt; Enum . min_max_by ( [ &quot;a&quot; , &quot;bcd&quot; , &quot;ef&quot; ] , &amp; String . length / 1 ) { &quot;a&quot; , &quot;bcd&quot; } At a known index ⚠️ WARNING Accessing linked lists by index is linear . If you are using the following in a nested call, you are probably doing something wrong. Raising on failure: Enum.fetch!/2 iex&gt; Enum . fetch! ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , 0 ) &quot;ant&quot; iex&gt; Enum . fetch! ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , - 1 ) &quot;cat&quot; iex&gt; Enum . fetch! ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , 3 ) ** (Enum.OutOfBoundsError) out of bounds error :ok / :error result: Enum.fetch/2 iex&gt; Enum . fetch ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , 2 ) { :ok , &quot;cat&quot; } iex&gt; Enum . fetch ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , 3 ) :error With default value: Enum.at/2 iex&gt; Enum . at ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , 3 ) nil iex&gt; Enum . at ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , 3 , &quot;none&quot; ) &quot;none&quot; Random sample Enum.random/1 iex&gt; Enum . random ( [ :a , :b , :c , :d ] ) :c","ref":"enum.html#return-search-a-single-element","title":"The Ultimate Enum Cheatsheet - Return/search a single element","type":"extras"},{"doc":"Count without stopping All elements: Enum.count/1 iex&gt; Enum . count ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] ) 3 Matching elements: Enum.count/2 iex&gt; Enum . count ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &amp; &amp;1 =~ &quot;at&quot; ) 2 Count up to a limit Enum.count_until/2 iex&gt; Enum . count_until ( 1 .. 20 , 5 ) 5 iex&gt; Enum . count_until ( 1 .. 20 , 50 ) 20 Matching elements: Enum.count_until/3 iex&gt; Enum . count_until ( 1 .. 20 , &amp; rem ( &amp;1 , 2 ) == 0 , 5 ) 5 iex&gt; Enum . count_until ( 1 .. 20 , &amp; rem ( &amp;1 , 2 ) == 0 , 50 ) 10 Other aggregations Enum.sum/1 iex&gt; Enum . sum ( [ 1 , 20 , 300 ] ) 321 Enum.product/1 iex&gt; Enum . product ( [ 1 , 20 , 300 ] ) 6000 Find an index Enum.find_index/2 iex&gt; Enum . find_index ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &amp; &amp;1 =~ &quot;at&quot; ) 1","ref":"enum.html#return-a-number","title":"The Ultimate Enum Cheatsheet - Return a number","type":"extras"},{"doc":"Sample data iex&gt; users = [ %{ id : 10 , name : &quot;Joe&quot; } , %{ id : 20 , name : &quot;Robert&quot; } , %{ id : 30 , name : &quot;Jose&quot; } ] From key/value pairs Map.new/2 / Map.new/1 iex&gt; Map . new ( users , &amp; { &amp;1 . id , &amp;1 . name } ) %{ 10 =&gt; &quot;Joe&quot; , 20 =&gt; &quot;Robert&quot; , 30 =&gt; &quot;Jose&quot; } iex&gt; Map . new ( [ { :foo , 12 } , { :bar , 5 } ] ) %{ bar : 5 , foo : 12 } Enum.into/3 / Enum.into/2 iex&gt; Enum . into ( users , %{ } , &amp; { &amp;1 . id , &amp;1 . name } ) %{ 10 =&gt; &quot;Joe&quot; , 20 =&gt; &quot;Robert&quot; , 30 =&gt; &quot;Jose&quot; } iex&gt; Enum . into ( [ { :foo , 12 } , { :bar , 5 } ] , %{ } ) %{ bar : 5 , foo : 12 } Aggregations Counting: Enum.frequencies/1 / Enum.frequencies_by/2 iex&gt; Enum . frequencies ( [ &quot;a&quot; , &quot;b&quot; , &quot;a&quot; , &quot;c&quot; ] ) %{ &quot;a&quot; =&gt; 2 , &quot;b&quot; =&gt; 1 , &quot;c&quot; =&gt; 1 } iex&gt; Enum . frequencies_by ( users , &amp; String . first ( &amp;1 . name ) ) %{ &quot;J&quot; =&gt; 2 , &quot;R&quot; =&gt; 1 } Grouping: Enum.group_by/2 / Enum.group_by/3 iex&gt; Enum . group_by ( users , &amp; String . first ( &amp;1 . name ) ) %{ &quot;J&quot; =&gt; [ %{ id : 10 , name : &quot;Joe&quot; } , %{ id : 30 , name : &quot;Jose&quot; } ] , &quot;R&quot; =&gt; [ %{ id : 20 , name : &quot;Robert&quot; } ] } iex&gt; Enum . group_by ( users , &amp; String . first ( &amp;1 . name ) , &amp; &amp;1 . id ) %{ &quot;J&quot; =&gt; [ 10 , 30 ] , &quot;R&quot; =&gt; [ 20 ] }","ref":"enum.html#return-a-map","title":"The Ultimate Enum Cheatsheet - Return a map","type":"extras"},{"doc":"Sample data words = [ &quot;hello&quot; , &quot;world&quot; ] Actually return a string Enum.map_join/3 iex&gt; Enum . map_join ( words , &quot;, &quot; , &amp; String . capitalize / 1 ) &quot;Hello, World&quot; When no need to transform: Enum.join/2 iex&gt; Enum . join ( 1 .. 5 , &quot;-&quot; ) &quot;1-2-3-4-5&quot; ⚠️ WARNING - Efficient string building Building your string manually using concatenation might lead to degraded performance, make sure to always rely on Enum.map_join/3 / Enum.join/2 or use IO data to build large strings dynamically. Building and using IO data Joining as IO data: Enum.map_intersperse/3 iex&gt; Enum . map_intersperse ( words , &quot;, &quot; , &amp; String . capitalize / 1 ) [ &quot;Hello&quot; , &quot;, &quot; , &quot;World&quot; ] When no need to transform: Enum.intersperse/2 # only when already list/enumerable of strings iex&gt; Enum . intersperse ( words , &quot;, &quot; ) [ &quot;hello&quot; , &quot;, &quot; , &quot;world&quot; ] # else need to map_intersperse + to_string iex&gt; Enum . map_intersperse ( 1 .. 5 , &quot;, &quot; , &amp; to_string / 1 ) [ &quot;1&quot; , &quot;, &quot; , &quot;2&quot; , &quot;, &quot; , &quot;3&quot; , &quot;, &quot; , &quot;4&quot; , &quot;, &quot; , &quot;5&quot; ] IO data to string: IO.iodata_to_binary/1 iex&gt; IO . iodata_to_binary ( [ &quot;Hello&quot; , &quot;, &quot; | [ &quot;World&quot; , ?! ] ] ) &quot;Hello, World!&quot; NOTE - Building a string might be un-necessary # IO data can be used directly for IO iex&gt; IO . puts ( [ &quot;Hello&quot; , &quot;, &quot; | [ &quot;World&quot; , ?! ] ] ) Hello , World ! :ok iex&gt; File . write! ( &quot;hello.txt&quot; , [ &quot;Hello&quot; , &quot;, &quot; | [ &quot;World&quot; , ?! ] ] ) :ok","ref":"enum.html#return-build-a-string","title":"The Ultimate Enum Cheatsheet - Return/build a string","type":"extras"},{"doc":"Matching a condition These will return early at the first match. Enum.any?/2 iex&gt; Enum . any? ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &amp; &amp;1 =~ &quot;at&quot; ) true iex&gt; Enum . any? ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &amp; &amp;1 =~ &quot;z&quot; ) false Enum.all?/2 iex&gt; Enum . all? ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &amp; &amp;1 =~ &quot;t&quot; ) true iex&gt; Enum . all? ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &amp; &amp;1 =~ &quot;at&quot; ) false Emptiness Enum.empty?/1 iex&gt; Enum . empty? ( [ ] ) true iex&gt; Enum . empty? ( [ :exists ] ) false Membership Enum.member?/2 iex&gt; Enum . member? ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &quot;bat&quot; ) true iex&gt; Enum . member? ( [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] , &quot;dog&quot; ) false Same but shorter: in/2 iex&gt; &quot;bat&quot; in [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; ] true Side note - Use a set: MapSet.new/1 / MapSet.new/2 Checking membership within a list has a linear cost - potentially quadratic within a loop. You might want to convert a list to a MapSet for efficient lookups. # DON&#39;T Enum . find ( fn x -&gt; x . id in list end ) # DO set = MapSet . new ( list ) Enum . find ( fn x -&gt; x . id in set end )","ref":"enum.html#return-a-boolean","title":"The Ultimate Enum Cheatsheet - Return a boolean","type":"extras"},{"doc":"Without transformation: Enum.concat/1 iex&gt; Enum . concat ( [ [ &quot;ant&quot; , &quot;bat&quot; ] , [ &quot;cat&quot; , &quot;dog&quot; ] ] ) [ &quot;ant&quot; , &quot;bat&quot; , &quot;cat&quot; , &quot;dog&quot; ] With transformation: Enum.flat_map/2 iex&gt; list = [ %{ data : [ 1 , 2 ] } , %{ data : [ 3 , 4 ] } ] iex&gt; Enum . flat_map ( list , &amp; &amp;1 . data ) [ 1 , 2 , 3 , 4 ]","ref":"enum.html#return-a-flattened-list","title":"The Ultimate Enum Cheatsheet - Return a flattened list","type":"extras"},{"doc":"By size: Enum.chunk_every/2 / Enum.chunk_every/4 iex&gt; Enum . chunk_every ( [ 1 , 2 , 3 , 4 , 5 , 6 ] , 2 ) [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] iex&gt; Enum . chunk_every ( [ 1 , 2 , 3 , 4 , 5 , 6 ] , 3 , 2 , :discard ) [ [ 1 , 2 , 3 ] , [ 3 , 4 , 5 ] ] Grouped by condition: Enum.chunk_by/2 iex&gt; animals = [ &quot;cat&quot; , &quot;bat&quot; , &quot;beaver&quot; , &quot;camel&quot; ] iex&gt; Enum . chunk_by ( animals , &amp; String . first / 1 ) [ [ &quot;cat&quot; ] , [ &quot;bat&quot; , &quot;beaver&quot; ] , [ &quot;camel&quot; ] ] See also: Enum.chunk_while/4","ref":"enum.html#return-several-smaller-lists","title":"The Ultimate Enum Cheatsheet - Return several smaller lists","type":"extras"},{"doc":"The closest thing to a for loop The most important one: Enum.reduce/3 iex&gt; Enum . reduce ( [ 2 , 4 , 8 ] , 0 , fn x , acc -&gt; ...&gt; acc + 1 / x ...&gt; end ) 0.875 # 1 / 2 + 1 / 4 + 1 / 8 ⚠️ WARNING - Building a list Lists should never be built by appending, always by prepending. Quoting the Erlang efficiency guide , &quot;To avoid copying the result in each iteration, build the list in reverse order and reverse the list when you are done&quot; : iex&gt; acc = Enum . reduce ( 1 .. 5 , [ ] , fn x , acc -&gt; [ x * * 2 | acc ] end ) [ 25 , 16 , 9 , 4 , 1 ] iex&gt; Enum . reverse ( acc ) [ 1 , 4 , 9 , 16 , 25 ] ⚠️ WARNING - Reinventing the wheel Enum.reduce/3 and its variants are very powerful and can be seen as the swiss army knife of the Enum module. But if there is a specialized function that does what you need, rolling your own manual implementation might be more verbose and less efficient (see the build a string section for example). Skipping the initial accumulator Enum.reduce/2 # first element will be used as accumulator instead iex&gt; Enum . reduce ( [ 2 , 4 , 8 ] , fn x , acc -&gt; ...&gt; acc + 1 / x ...&gt; end ) 2.375 # 2 + 1 / 4 + 1 / 8 ⚠️ WARNING - Edge cases # It might seem that 0 is optional here... iex&gt; Enum . reduce ( [ 10 , 20 , 30 ] , &amp; + / 2 ) 60 # ... but will actually raise if empty iex&gt; Enum . reduce ( [ ] , &amp; + / 2 ) ** (Enum.EmptyError) empty error Early returns (like break ) Enum.reduce_while/3 iex&gt; Enum . reduce_while ( [ 2 , 4 , 0 , 1 ] , 0 , fn x , acc -&gt; ...&gt; if x == 0 do ...&gt; { :halt , acc } ...&gt; else ...&gt; { :cont , acc + 1 / x } ...&gt; end ...&gt; end ) 0.75 # 1 / 2 + 1 / 4 NOTE - Recursion The benefit of Enum.reduce_while/3 is that it works with any enumerable. However, a recursion-based implementation might be more readable, maintainable and performant if you only need to support lists. The previous example could be re-implemented as: def sum_inverses ( [ x | xs ] , acc ) when x != 0 do sum_inverses ( xs , acc + 1 / x ) end def sum_inverses ( _ , acc ) , do : acc Working with several lists / enumerables Two inputs: Enum.zip_reduce/4 iex&gt; Enum . zip_reduce ( [ 3 , 4 , 2 ] , [ 100 , 10 , 1 ] , 0 , fn x , y , acc -&gt; ...&gt; x * y + acc ...&gt; end ) 342 # 3 * 100 + 4 * 10 + 2 * 1 More inputs: Enum.zip_reduce/3 iex&gt; Enum . zip_reduce ( [ [ 4 , 2 ] , [ 10 , 1 ] , [ 1 , - 1 ] ] , 0 , fn xs , acc -&gt; ...&gt; Enum . product ( xs ) + acc ...&gt; end ) 38 # 4 * 10 * 1 + 2 * 1 * -1 Keeping all the steps With initial accumulator: Enum.scan/3 iex&gt; Enum . scan ( [ 2 , 4 , 8 ] , 0 , fn x , acc -&gt; ...&gt; acc + 1 / x ...&gt; end ) [ 0.5 , 0.75 , 0.875 ] Without initial accumulator: Enum.scan/2 iex&gt; Enum . scan ( [ 2 , 4 , 8 ] , fn x , acc -&gt; ...&gt; acc + 1 / x ...&gt; end ) [ 2 , 2.25 , 2.375 ]","ref":"enum.html#when-nothing-else-works","title":"The Ultimate Enum Cheatsheet - When nothing else works","type":"extras"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#dev","title":"Changelog - Dev","type":"extras"},{"doc":"Enhancements Add Iter.take_every/2 and Iter.drop_every/2","ref":"changelog.html#v0-1-2-2022-12-28","title":"Changelog - v0.1.2 (2022-12-28)","type":"extras"},{"doc":"Enhancements Add Iter.slice/2 and Iter.slice/3","ref":"changelog.html#v0-1-1-2022-12-26","title":"Changelog - v0.1.1 (2022-12-26)","type":"extras"},{"doc":"Initial release","ref":"changelog.html#v0-1-0-2022-12-25","title":"Changelog - v0.1.0 (2022-12-25)","type":"extras"},{"doc":"MIT License Copyright (c) 2022 Sabiwara Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"}]